//***************************************************************************************************
// RC5 Empfnger fr den Serialport, Girder kompatibel sowie Tester-Modus
// Autor: Christian Richter
// Basierend auf den TI-Application Notes fr RC5 Empfang und Software-UART
// Verwendeter Prozessor: MSP430F1121, es kann jedoch jeder MSP430 mit Timer A verwendet werden.
// Es werden alle 3 CCR des Timer A verwendet.
// Frei fr private Nutzung
//***************************************************************************************************

#include "project.h"

volatile uint8_t i;
extern volatile unsigned SYSTEM_FLAGS;
extern volatile unsigned char RX_Bytes[16];
extern volatile unsigned char charcount;
volatile uint8_t Modus;
extern volatile uint16_t IRData;

volatile uint8_t Command;
volatile uint8_t Address;
volatile uint8_t Toggle;

volatile uint8_t LastCommand;
volatile uint8_t LastAddress;
volatile uint8_t LastToggle;

int main(void)
{
	//Watchdog deaktivieren
	WDTCTL = WDTHOLD + WDTPW;

	//XT1 aktivieren und auf Stabilitt testen
	BCSCTL1 |= XTS;                           // ACLK = LFXT1 = HF XTAL
	do
	{
	IFG1 &= ~OFIFG;                           // Clear OSCFault flag
	for (i = 0xFF; i > 0; i--);               // Time for flag to set
	}
	while ((IFG1 & OFIFG));                   // OSCFault flag still set?

	BCSCTL2 |= SELM_3;                        // MCLK = LFXT1 (safe)

	Modus = Girder;

	CCTL0 = OUT;                              // TXD Idle as Mark
	TACTL = TASSEL_1 + MC_2 + TACLR + ID_3;   // SMCLK, continuous mode, Teiler :8 -> TACLK = 460,8kHz

	P1DIR = 0x03;		//P1.0 AUsgang LED, P1.1 Ausgang Serial Data
	P1OUT &= ~0x01;		//LED ausschalten
	P1SEL = 0x02;		//P1.1 ist Timer A0 Ausgang Software UART TX
	P2SEL = 0x04;		//P2.2 ist Timer A0 Eingang Software UART RX

	_EINT();			//Globalen Interrupt freigeben, sonst weder Senden noch Empfangen per Software UART

	RX_Ready();			//Serial-Empfang ein
	IR_Ready();			//IR-Empfang ein

	while(1)
	{
		if(SYSTEM_FLAGS)	//Was zu tun?
		{
			if(SYSTEM_FLAGS & UART_RX_FLAG)		//Zeichen vom PC empfangen
			{
				if(charcount == 2)
				{
					if((RX_Bytes[0] == 'r') && (RX_Bytes[1] == 'c')) //Fernbedien-Test Modus ASCII Ausgabe
					{
						Modus = Tester;
						_EINT();
						SendString("RC5 Tester Modus\r\n");

					}
					if((RX_Bytes[0] == 'I') && (RX_Bytes[1] == 'R'))	//Girder Modus, 6 Bytes Binr Ausgabe
					{
						Modus = Girder;
						_EINT();
						SendString("OK");			//Besttigung an Girder / EventGhost
					}
					charcount = 0;

				}
				SYSTEM_FLAGS &= ~UART_RX_FLAG;
				RX_Ready();							//Bereit fr neue Zeichen
			}

			if(SYSTEM_FLAGS & IR_RX_FLAG)			//Ein kompletter RC5-Code wurde empfangen
			{
				P1OUT |= 0x01;						//Besttigungs LED ein

				Command = (uint8_t)IRData & 0x3F;			//Dekodieren
				Address = (uint8_t)((IRData >> 6) & 0x1F);
				Toggle = (uint8_t)((IRData >> 11) & 0x01);
				if(Modus == Girder)
				{

					//Nur an Girder weiter geben, wenn neuer Tastendruck auf gleihe Taste erfolgte
					if((LastCommand == Command) && (LastAddress == Address))
					{
						if(LastToggle != Toggle)	//Prfung auf nderung des Toggle-Bits
						{
							putchar(Address);
							putchar(Command);
							putchar(0);
							putchar(0);
							putchar(0);
							putchar(0);
						}
					}
					else		//Direkte Ausgabe, wenn neue Taste gedrckt
					{
						putchar(Address);
						putchar(Command);
						putchar(0);
						putchar(0);
						putchar(0);
						putchar(0);
					}
					RX_Ready();
				}
				else	//Ausgabe im Tester-Modus formatiert fr Hyperterminal o..
				{
					if((LastCommand == Command) && (LastAddress == Address))
					{
						SendString("Toggle-Bit: ");
						putchar(Toggle + 0x30);
						SendString(" Address: ");
						putchar((Address / 10) + 0x30);
						putchar((Address % 10) + 0x30);
						SendString(" Command: ");
						putchar((Command / 10) + 0x30);
						putchar((Command % 10) + 0x30);
						SendString("\r\n");

						RX_Ready();
					}
				}

				//Letzten Tastendruck speichern

				LastCommand = Command;
				LastAddress = Address;
				LastToggle = Toggle;

				SYSTEM_FLAGS &= ~IR_RX_FLAG;
				//Etwas warten, weil der PC die Kommandos sonst verschluckt
				Wait(40000);
				P1OUT &= ~0x01;	//LED aus
				IR_Ready();		//Bereit fr neues IR-Kommando

				_EINT();		// Int wieder ein

			}
		}
		_EINT();
	}

	return 0;
}
